On Fri, Oct 26 2007, Sebastian Siewior wrote:
> * Jens Axboe | 2007-10-26 11:32:42 [+0200]:
> 
> >On Fri, Oct 26 2007, Jens Axboe wrote:
> >> > 
> >> > I hope this was usefull. Now, I'm going to rebuild my raid now....
> >> > 
> >> Thanks a lot, a full report on this issue. Will get this fixed up asap.
> No problem, thanks for working on that :)
> 
> >
> >Does this work?
> >
> >diff --git a/drivers/scsi/scsi_lib.c b/drivers/scsi/scsi_lib.c
> >index 61fdaf0..cf47fcb 100644
> >--- a/drivers/scsi/scsi_lib.c
> >+++ b/drivers/scsi/scsi_lib.c
> >@@ -1115,6 +1115,8 @@ static int scsi_init_io(struct scsi_cmnd *cmd)
> >      * kmapping pages)
> >      */
> >     cmd->use_sg = req->nr_phys_segments;
> >+    if (!cmd->use_sg)
> >+            return 0;
> > 
> >     /*
> >      * If sg table allocation fails, requeue request later.
> >@@ -1191,7 +1193,7 @@ int scsi_setup_blk_pc_cmnd(struct scsi_device *sdev, 
> >struct request *req)
> >     if (req->bio) {
> >             int ret;
> > 
> >-            BUG_ON(!req->nr_phys_segments);
> >+            BUG_ON(!req->nr_phys_segments && req->bio->bi_size);
> > 
> >             ret = scsi_init_io(cmd);
> >             if (unlikely(ret))
> >
> 
> Nope. I get [1] on manual umount and [2] on system reboot. This is
> 24-rc1 with this patch on top.
> 
> [1] http://download.breakpoint.cc/bug/bug_rc1_patch_manual.jpeg 163 KiB
> [2] http://download.breakpoint.cc/bug/bug_rc1_patch_reboot.jpeg 171 KiB

Ah, second BUG() for same issue. Try this one. This?

diff --git a/drivers/scsi/scsi_lib.c b/drivers/scsi/scsi_lib.c
index 61fdaf0..57fde7b 100644
--- a/drivers/scsi/scsi_lib.c
+++ b/drivers/scsi/scsi_lib.c
@@ -1115,6 +1115,8 @@ static int scsi_init_io(struct scsi_cmnd *cmd)
         * kmapping pages)
         */
        cmd->use_sg = req->nr_phys_segments;
+       if (!cmd->use_sg)
+               return 0;
 
        /*
         * If sg table allocation fails, requeue request later.
@@ -1191,7 +1193,7 @@ int scsi_setup_blk_pc_cmnd(struct scsi_device *sdev, 
struct request *req)
        if (req->bio) {
                int ret;
 
-               BUG_ON(!req->nr_phys_segments);
+               BUG_ON(!req->nr_phys_segments && req->bio->bi_size);
 
                ret = scsi_init_io(cmd);
                if (unlikely(ret))
@@ -1236,9 +1238,10 @@ int scsi_setup_fs_cmnd(struct scsi_device *sdev, struct 
request *req)
        if (ret != BLKPREP_OK)
                return ret;
        /*
         * Filesystem requests must transfer data.
         */
-       BUG_ON(!req->nr_phys_segments);
+       BUG_ON(!req->nr_phys_segments && !bio_empty_barrier(req->bio));
 
        cmd = scsi_get_cmd_from_req(sdev, req);
        if (unlikely(!cmd))
@@ -1236,9 +1238,10 @@ int scsi_setup_fs_cmnd(struct scsi_device *sdev, struct 
                return ret;
        /*
-        * Filesystem requests must transfer data.
+        * Filesystem requests must transfer data, unless it's an empty
+        * barrier.
         */

-- 
Jens Axboe

-
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to [EMAIL PROTECTED]
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Reply via email to